/*
 * SDLKobo v0.1
----------------------------------------------------------------------
	gfxengine.h - Graphics Engine
----------------------------------------------------------------------
 *  David Olofson, 2001
 * This code is released under the terms of the GNU GPL.
 */

#ifndef	_GFXENGINE_H_
#define	_GFXENGINE_H_

#define GFX_BANKS	256


extern "C"
{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
}
#include "SDL.h"

#include "sprite.h"
#include "cs.h"

typedef enum
{
	SINGLE = 0,
	DOUBLE,
	HALF
} buffermodes_t;

typedef enum
{
	FLIP = 0,
	TIMER,
	RETRACE
} syncmodes_t;

class window_t;
class SoFont;

class gfxengine_t
{
  public:
	gfxengine_t();
	virtual ~gfxengine_t();

	void output(window_t *outwin);
	window_t *output()	{ return window; }

	/* Initialization */
	void size(int w, int h);
	void mode(int bits, int fullscreen);
	void buffering(buffermodes_t bm, syncmodes_t sm);
	void period(float frameduration); // 0 to reset internal timer
	void wrap(int x, int y);
	void interpolation(int inter);

	/* Engine open/close */
	int open(int objects = 1024);
	void close();

	/* Data management (use while engine is open) */
	void colorkey(int r, int g, int b);
	int loadimage(int bank, const char *name);
	int loadsprites(int bank, int w, int h, const char *name);
	int loadfont(int bank, const char *name);
	void reload();
	void unload(int bank);

	/* Settings (use while engine is open) */
	void title(const char *win, const char *icon);

	/* Display show/hide */
	int show();
	void hide();

	/* Main loop take-over */
	void run();

	/*
	 * Override these;
	 *	frame() is called once per control system frame,
	 *		after the control system has executed.
	 *	pre_render() is called after the engine has advanced
	 *		to the state for the current video frame
	 *		(interpolated state is calculated), before
	 *		the engine renders all graphics.
	 *	post_render() is called after the engine have
	 *		rendered all sprites, but before video the
	 *		sync/flip/update operation.
	 */
	virtual void frame();
	virtual void pre_render();
	virtual void post_render();

	/*
	---------------------------------------------
	 * The members below this line can safely be
	 * called from within the frame() handler.
	 */

	/* Control */
	cs_engine_t *cs()	{ return csengine; }
	SDL_Surface *surface()	{ return screen; }
	void stop();
	cs_obj_t *get_obj(int layer);
	void free_obj(cs_obj_t *obj);
	void cursor(int csr);

	SDL_Surface *get_sprite(int bank, int frame);
	SoFont *get_font(unsigned int f);

	void scroll_ratio(int layer, float xr, float yr);
	void scroll(int xs, int ys);
	void force_scroll();

	int xoffs(int layer);
	int yoffs(int layer);

	/* Info */
	int objects_in_use();

  protected:
	SDL_Surface	*screen;
	window_t	*window;
	int		wx, wy;
	int		xscroll, yscroll;
	float		xratio[CS_LAYERS];
	float		yratio[CS_LAYERS];
	s_container_t	*gfx;
	SoFont		*fonts[GFX_BANKS];	/* Kludge. */
	cs_engine_t	*csengine;
	buffermodes_t	buffermode;
	syncmodes_t	syncmode;
	int		_fullscreen;
	int		use_interpolation;
	int		_width, _height;
	int		_depth;
	const char	*_title;
	const char	*_icontitle;
	int		_cursor;

	int		start_tick;
	float		ticks_per_frame;

	int		is_showing;
	int		is_running;
	int		is_open;

	static void on_frame(cs_engine_t *e);
	void __frame();

	void sync_half();
	void sync();
	void flip();
	void start_engine();
	void stop_engine();
	static void render_sprite(cs_obj_t *o);
};


extern gfxengine_t *gfxengine;

#endif
