/*
------------------------------------------------------------
	window.h - Generic Rendering Window
------------------------------------------------------------
 *  David Olofson, 2001
 * This code is released under the terms of the GPL.
 */

#ifndef _WINDOW_H_
#define _WINDOW_H_

#include "SDL.h"

class gfxengine_t;

class window_t
{
	friend class gfxengine_t;
  public:
	window_t();
	virtual ~window_t();

	void init(gfxengine_t *e);
	void place(int left = 0, int top = 0,
			int sizex = 200, int sizey = 200,
			int vsizex = -1, int vsizey = -1);
	void select();

#if 0
	void set_position(int vposx, int vposy);
	void force_position(int vposx, int vposy);

	void get_visible_rect(SDL_Rect *r);
	void window_to_screen(SDL_Rect *r);
	void window_to_screen_fxp(int &x, int &y);
#endif

	/* Rendering */
	Uint32 alloc_color(int r, int g, int b);
	void foreground(Uint32 color);
	void background(Uint32 color);

	void clear();
	void font(int fnt);
	void string(int x, int y, const char *txt);
	void center(int y, const char *txt);

	void point(int x, int y);
	void fillrect(int x, int y, int w, int h);

	void sprite(int x, int y, int bank, int frame);

	int x()		{ return rect.x; }
	int y()		{ return rect.y; }
	int width()	{ return rect.w; }
	int height()	{ return rect.h; }
	int x2()	{ return rect.x + rect.w; }
	int y2()	{ return rect.y + rect.h; }

  protected:
	/*
	 * Kludge: This is for window selection to work.
	 * the problem is that this will break (along
	 * with some other things) if more than one
	 * gfxengine_t instance is used.
	 */
	static window_t	*selected;	/* Selected */

	gfxengine_t	*engine;
	SDL_Rect	rect;
	int		vx, vy;
	int		gvx, gvy;
	int		__gvx, __gvy;
	int		vw, vh;
	Uint32		fgcolor, bgcolor;
	int		_font;
};

#endif
