/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#undef	DEBUG_OUT

extern "C"
{
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
}

#include "skobo.h"
#include "config.h"
#include "tiff.h"
#include "screen.h"
#include "manage.h"
#include "score.h"
#include "key.h"
#include "random.h"
#include "version.h"
#include "sound/sounds.h"
#include "sound/bgmfile.h"

skobo_gfxengine_t gengine;

window_t wbase;
window_t wchip;
window_t wradar;
//window_t wscore;

display_t dhigh;
display_t dscore;
display_t dstage;
display_t dships;

int mouse_x = 0;
int mouse_y = 0;
int cheat_mode = 0;
int indicator = 0;
int use_special_bgm = 0;
int sound = 1;


void skobo_gfxengine_t::frame()
{
	SDL_Event ev;
	while(SDL_PollEvent(&ev))
	{
		switch(ev.type)
		{
		  case SDL_KEYDOWN:
			key.press(ev.key.keysym.sym);
			break;
		  case SDL_KEYUP:
			key.release(ev.key.keysym.sym);
			break;
		}
	}
	if(manage.mainloop())
		stop();
}


void skobo_gfxengine_t::pre_render()
{
	_screen::pre_render(&wchip);
}


void skobo_gfxengine_t::post_render()
{
	_screen::post_render(&wchip);
#ifndef USE_NO_FRAME
	wchip.sprite(0, 0, B_FRAME, 0);
#endif
#ifdef DEBUG_OUT
	char buf[100];
	snprintf(buf, 100, "Obj: %d", gengine.objects_in_use());
	wchip.center(0, buf);
#endif
}


void myexit(void)
{
	gengine.stop();
	if(sound)
		audio_close();
	SDL_Quit();
}


void breakhandler(int dummy)
{
	exit(1);
}


#if 0
void motion_callback(win &)
{
	mouse_x = win::xevt.xmotion.x;
	mouse_y = win::xevt.xmotion.y;
	mouse_x -= (WSIZE >> (1 - scale_log2));
	mouse_y -= (WSIZE >> (1 - scale_log2));
	if ((mouse_x == 0) && (mouse_y == 0))
		return;
	key.mouse_position(mouse_x, mouse_y);
}

void leave_callback(win &)
{
	mouse_x = 0;
	mouse_y = 0;
}

void focus_in_callback(win &)
{
	key.clear();
}

void focus_out_callback(win &)
{
	key.clear();
}

void button_press_callback(win &)
{
	key.mouse_press(win::xevt.xbutton.button);
}

void button_release_callback(win &)
{
	key.mouse_release(win::xevt.xbutton.button);
}

void client_message_callback(win &)
{
	Atom wm_protocols, wm_delete_window;
	wm_protocols = XInternAtom(win::disp, "WM_PROTOCOLS", False);
	wm_delete_window =
		XInternAtom(win::disp, "WM_DELETE_WINDOW", False);
	if ((win::xevt.xclient.message_type == wm_protocols)
		&& (win::xevt.xclient.data.l[0] == wm_delete_window))
	{
		_manage::exit_key();
		_manage::exit_key();
	}
}
#endif

void put_usage()
{
	printf("\nSKOBO %s\n", KOBO_VERSION);
	printf("Usage: skobo -hiscores\n");
	printf("       skobo [-fullscreen]\n");
	printf("             [-depth XX]\n");
	printf("             [-nofilter]\n");
	printf("             [-cheat]\n");
	printf("             [-indicator]\n");
	printf("             [-wait XXX] (default is %d)\n", WAIT_MSEC);
	printf("             [-nosound]\n");
	printf("             [-bgm BGMINDEX]\n");
	printf("             [-vol XXX]  (default is 100)\n");
	printf("\n");
	exit(1);
}

void put_score()
{
	scorefile.show_high_scores(1);
	exit(0);
}

/*           xkobo          */

int main(int argc, char *argv[])
{
	int i;
	char *bgm_indexfile = NULL;
	int wait_msec = WAIT_MSEC;
	int use_fullscreen = 0;
	int depth = 0;	//use default/current
	int filter = 1;	//use "motion filtering"

	atexit(myexit);
	signal(SIGTERM, breakhandler);
	signal(SIGINT, breakhandler);

	for (i = 1; i < argc; i++)
	{
		if (!strcmp("-fullscreen", argv[i]))
			use_fullscreen = 1;
		else if (!strcmp("-depth", argv[i]))
			depth = atoi(argv[++i]);
		else if (!strcmp("-nofilter", argv[i]))
			filter = 0;
		else if (!strcmp("-cheat", argv[i]))
			cheat_mode = 1;
		else if (!strcmp("-indicator", argv[i]))
			indicator = 1;
		else if (!strcmp("-wait", argv[i]))
			wait_msec = atoi(argv[++i]);
		else if (!strcmp("-hiscores", argv[i]))
			put_score();
		else if (!strcmp("-nosound", argv[i]))
			sound = 0;
		else if (!strcmp("-bgm", argv[i]))
			bgm_indexfile = argv[++i];
		else if (!strcmp("-vol", argv[i]))
			audio_master_volume(atoi(argv[++i]));
		else
			put_usage();
	}

	gengine.size(SCREEN_WIDTH, SCREEN_HEIGHT);
	gengine.mode(0, use_fullscreen);
	gengine.buffering(DOUBLE, FLIP);
	gengine.period(wait_msec);
	gengine.cursor(0);
	gengine.interpolation(filter);
	gengine.scroll_ratio(LAYER_OVERLAY, 0.0, 0.0);
	gengine.scroll_ratio(LAYER_PLAYER, 1.0, 1.0);
	gengine.scroll_ratio(LAYER_ENEMIES, 1.0, 1.0);
	gengine.scroll_ratio(LAYER_BASES, 1.0, 1.0);
	gengine.scroll_ratio(LAYER_GROUND, 1.0, 1.0);
	gengine.scroll_ratio(LAYER_STARS, 0.5, 0.5);
	gengine.open(2048);

	gengine.wrap(MAP_SIZEX*CHIP_SIZEX, MAP_SIZEY*CHIP_SIZEY);

	gengine.colorkey(65535, 0, 0);
	if(gengine.loadsprites(B_GRAPHICS, 16, 16, "sprites.pcx") < 0)
		return 1;
	if(gengine.loadsprites(B_BULLETS, 8, 8, "bullets.pcx") < 0)
		return 1;
	if(gengine.loadsprites(B_BIGSHIP, 32, 32, "bigship.pcx") < 0)
		return 1;
#ifndef USE_NO_FRAME
#  ifdef	USE_NICE_FRAME
	if(gengine.loadimage(B_FRAME, "frame.png") < 0)
		return 1;
#  else
	if(gengine.loadsprites(B_FRAME, WSIZE, WSIZE, "frame.gif") < 0)
		return 1;
#  endif
#endif
	if(gengine.loadimage(B_SCREEN, "screen.png") < 0)
		return 1;
	if(gengine.loadimage(B_LOGO, "logo-ng-yellow.png") < 0)
		return 1;

	gengine.colorkey(0, 0, 0);
	if(gengine.loadfont(B_NORMAL_FONT, "font2s.png") < 0)
		return 2;

	gengine.title("SKobo v0.1", "skobo");

	wbase.init(&gengine);
	wchip.init(&gengine);
	wradar.init(&gengine);

	dhigh.init(&gengine);
	dscore.init(&gengine);
	dstage.init(&gengine);
	dships.init(&gengine);

	wbase.place(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);

	wchip.place(8+MARGIN, MARGIN, WSIZE, WSIZE,
			MAP_SIZEX*CHIP_SIZEX, MAP_SIZEY*CHIP_SIZEY);

	dhigh.place(8 + 240 + 1, 8, 64, 18);
	dscore.place(dhigh.x(), dhigh.y2() + 6, 64, 18);

	wradar.place(dhigh.x(), (SCREEN_HEIGHT-MAP_SIZEY)/2,
			MAP_SIZEX, MAP_SIZEY);

	dships.place(dhigh.x(), wradar.y2() + 6, 38, 18);
	dstage.place(dhigh.x(), dships.y2() + 6, 38, 18);

	wbase.font(B_NORMAL_FONT);
	wbase.foreground(wbase.alloc_color(65535, 65535, 65535));
	wbase.background(wbase.alloc_color(0x4600, 0x5800, 0x6500));
	wbase.sprite(0, 0, B_SCREEN, 0);

	wchip.font(B_NORMAL_FONT);
	wchip.foreground(wbase.alloc_color(65535, 65535, 65535));
	wchip.background(wbase.alloc_color(0, 0, 0));
	wchip.clear();
	gengine.output(&wchip);

//	Uint32 dc = wbase.alloc_color(0x4600*3>>2, 0x5800*3>>2, 0x6500*3>>2);
	Uint32 dc = wbase.alloc_color(0, 0x3333, 0x6666);

	dhigh.font(B_NORMAL_FONT);
	dhigh.color(dc);
	dhigh.caption("HIGHSCORE");

	dscore.font(B_NORMAL_FONT);
	dscore.color(dc);
	dscore.caption("SCORE");

	dships.font(B_NORMAL_FONT);
	dships.color(dc);
	dships.caption("SHIPS");

	dstage.font(B_NORMAL_FONT);
	dstage.color(dc);
	dstage.caption("STAGE");

	wradar.foreground(wbase.alloc_color(65535, 65535, 65535));
	wradar.background(wbase.alloc_color(0, 0, 0));
	wradar.clear();


	rand_num.init();

	if(sound)
	{
		if(bgm_indexfile)
		{
			init_bgm(bgm_indexfile);
			use_special_bgm = 1;
		}
		if(audio_open() < 0)
		{
			fprintf(stderr, "Couldn't initialize audio;"
					" disabling sound effects.\n");
			sound = 0;
		}
	}
	init_sounds();

	manage.init();

	gengine.run();

	gengine.close();

	if(sound)
		audio_close();

	return 0;
}
